# Importing external dependencies
import sys

# Importing internal classes
from ..storage import Storage

def store_to_csv(csv_obj=Storage.CsvStorageContext(None), data=None):
    
    original_stdout = sys.stdout

    with open(csv_obj.file_location, 'a') as f:
        sys.stdout = f
        print(data + ',')
        sys.stdout = original_stdout


# Importing external libraries
import sys
import unittest

# Importing libraries we want to test
sys.path.append('../../')
import src.gqpy.gqpy as gq

class TestSymbols(unittest.TestCase):

    def setUp(self):
        self.gq = gq.GoQuant()

    def test_binance(self):

        self.get_symbols('binance')

    def test_bitfinex(self):

        self.get_symbols('bitfinex')

    def test_coinbase(self):

        self.get_symbols('coinbase')

    def test_kraken(self):

        self.get_symbols('kraken')

    def get_symbols(self, exchange_name):

        res = self.gq.symbols.get(exchange_name)
        if exchange_name == 'kraken':
            print(res)
        self.assertTrue(type(res) == list and len(res) > 0) 



if __name__ == "__main__":
    unittest.main()


# Importing external libraries
import requests

# Importing internal utility functions
from ..symbols.format import format_symbols
from ..storage.storage_utils import store_data



class Static:

    def __init__(self, url):
        self.url = url

    def ohlcv(self, exchange_name, symbol_pair, storage_obj=None):
        return handle_static_data(self.url, exchange_name, symbol_pair, 'ohlcv', storage_obj)

    def orderbook(self, exchange_name, symbol_pair, storage_obj=None):
        return handle_static_data(self.url, exchange_name, symbol_pair, 'orderbook_l2', storage_obj)

    def quote(self, exchange_name, symbol_pair, storage_obj=None):
        return handle_static_data(self.url, exchange_name, symbol_pair, 'quote', storage_obj)

    def trades(self, exchange_name, symbol_pair, limit, storage_obj=None):
        return handle_static_data(self.url, exchange_name, symbol_pair, 'trades', storage_obj, limit)

def handle_static_data(url, exchange_name, symbol_pair, data_type, storage_obj, limit=None):

    # Connecting to API V1
    connection_url = f'{url}api/static/{exchange_name}/spot?data_type={data_type}&symbol={symbol_pair.upper()}'

    if data_type == 'trades':
        connection_url += f'&limit={limit}'
    
    res = requests.get(connection_url).json()

    # If storage object is None, then the response is just returned
    if storage_obj:
        store_data(storage_obj, res)

    return res


# Importing external libraries
import unittest

# Importing libraries we want to test
from tests import Tests

class TestAllExchangesLive(unittest.TestCase):

    def setUp(self):
        
        self.tests = Tests()

    def test_binance(self):

        self.tests.ticker('binance', ['btcusdt'], 'BTCUSDT', False, 5)
        self.tests.book('binance', ['btcusdt'], 'BTCUSDT', False, 5)
        self.tests.trade('binance', ['btcusdt'], 'BTCUSDT', False, 5)

    def test_bitfinex(self):

        self.tests.ticker('bitfinex', ['BTCUSD'], 'tBTCUSD', False, 5)
        self.tests.book('bitfinex', ['BTCUSD'], 'tBTCUSD', False, 5)
        self.tests.trade('bitfinex', ['BTCUSD'], 'tBTCUSD', False, 5)

    def test_coinbase(self):

        self.tests.ticker('coinbase', ['BTC-USD'], 'BTC-USD', False, 5)
        self.tests.book('coinbase', ['BTC-USD'], 'BTC-USD', False, 5)
        self.tests.trade('coinbase', ['BTC-USD'], 'BTC-USD', False, 5)

    def test_kraken(self):

        self.tests.ticker('kraken', ['XBT/USD'], 'XBT/USD', False, 5)
        self.tests.book('kraken', ['XBT/USD'], 'XBT/USD', False, 5)
        self.tests.trade('kraken', ['XBT/USD'], 'XBT/USD', False, 5)

if __name__ == "__main__":
    unittest.main()


# Internal libraries
import requests

class Symbols():

    def get(self, exchange_name):
        res = requests.get(f'https://api.goquant.io/symbols?exchange={exchange_name}')
        res = res.text[1:-1].split(',')
        res = [i.replace('"', '', 2) for i in res]
        return res



# Importing internal functionality
from .handler import handle_live_data

class Live:

    def __init__(self, url):
        self.url = url
        self.__supported_exchanges = ['binance', 'bitinfex', 'coinbase', 'kraken']

    # By default, output will be printed, the WS will run indefinitely and it will not be stored anywhere
    def book(self, exchange_name, symbol_pair, print=True, run_time=True, storage_obj=None):
        return handle_live_data(self.url, exchange_name, symbol_pair, print, run_time, 'book', storage_obj)

    def ticker(self, exchange_name, symbol_pair, print=True, run_time=True, storage_obj=None):
        return handle_live_data(self.url, exchange_name, symbol_pair, print, run_time, 'ticker', storage_obj)

    def trade(self, exchange_name, symbol_pair, print=True, run_time=True, storage_obj=None):
        return handle_live_data(self.url, exchange_name, symbol_pair, print, run_time, 'trade', storage_obj)

    def get_supported_exchanges(self):
        return self.__supported_exchanges


# TODO: Remove this file, it is no longer used for symbol normalization

# Importing external libaraies
import sys

# Importing internal utility functions
from .requests import binance, bitfinex, bitstamp, bybit, coinbase, huobi, okex



def get_symbols(exchange_name):

    exchange_name = exchange_name.lower()
    symbols = []

    match exchange_name:
        case 'binance':
            symbols = binance()
        case 'bitfinex':
            symbols = bitfinex()
        case 'bitstamp':
            return bitstamp()
        case 'bybit':
            symbols = bybit()
        case 'coinbase':
            symbols = coinbase()
        case 'huobi':
            symbols = huobi()
        case 'okex':
            symbols = okex()

    return clean_symbols(symbols)
    
def clean_symbols(available_symbols):

    clean_list = []
    for s in available_symbols:

        # All symbol pairs will be standardized to the following format: aaabbb
        clean_list.append(s.lower().replace('-', '').replace('/', '').replace(' ', ''))

    return clean_list

def format_symbols(exchange_name, symbol_1, symbol_2):

    symbol_1 = symbol_1.lower()
    symbol_2 = symbol_2.lower()

    available_symbols = get_symbols(exchange_name)

    # Exchanges with dashed format (aaa-bbb)
    dashed_format = ['coinbase']

    # Exchanges with slashed format (aaa/bbb)
    slashed_format = ['kraken']

    # Exchanges with no dash
    regular_format = ['binance', 'bitfinex']

    # Getting the seperator for the exchange
    seperator = None
    if exchange_name in dashed_format:
        seperator = '-'
    elif exchange_name in slashed_format:
        seperator = '/'
    elif exchange_name in regular_format:
        seperator = ''
    else:
        print("The exchange entered is not a supported exchange.")
        sys.exit(1)

    # Bitfinex uses USD instead of USDT
    if exchange_name == 'bitfinex' and symbol_1.lower() == 'usdt':
        symbol_1 = 'usd'
    elif exchange_name == 'bitfinex' and symbol_2.lower() == 'usdt':
        symbol_2 = 'usd'

    symbol_pair = ''

    # Concatenating the two symbol pairs
    if symbol_1 + symbol_2 in available_symbols:
        symbol_pair = symbol_1 + seperator + symbol_2
    elif symbol_2 + symbol_1 in available_symbols:
        symbol_pair = symbol_2 + seperator + symbol_1
    else:
        print("That symbol pair is not supported by the specified exchange.")
        sys.exit(1)

    return symbol_pair


# Importing external libraries
import websocket
import json
import time
import sys

# Importing internal utility functions
from ...storage.storage_utils import store_data

def handle_live_data(url, exchange_name, symbol_pairs, print_recv, run_time, data_type, storage_obj):

    ws = websocket.WebSocket()
    ws.connect(url)

    # Allowing multiple connections to be made
    for pair in symbol_pairs:
        connection_string = f'{{"event":"subscribe", "channel":"{exchange_name.lower()}.spot.{data_type}.{pair}"}}'
        ws.send(connection_string)

    start_time = time.time()
    cache = []

    while True:

        if type(run_time) != bool:

            current_time = time.time()
            elapsed_time = current_time - start_time

            if elapsed_time > run_time:
                # print(f'Data pipeline ran successfully and exited after {str(run_time)} seconds.')
                ws.close()
                return cache

        try:
            
            msg = ws.recv()
            
            # Checking to see if the symbol pair is supported
            if msg == '{"error":"Unsupported symbol pair."}':
                print(f'That symbol pair is not supported by {exchange_name}.')
                sys.exit(1)
            
            cache.append(json.loads(msg))

            if storage_obj:
                store_data(storage_obj, msg)
            elif print_recv:
                print(json.loads(msg))

        except KeyboardInterrupt:

            ws.close()
            break


import unittest
import sys
import random

# Importing libraries we want to test
sys.path.append('../../')
import src.gqpy.gqpy as gq

class Tests(unittest.TestCase):

    def ticker(self, exchange, input_symbol_pairs, output_symbol_pairs, print_res, time):

        gqpy = gq.GoQuant()

        res = gqpy.live.ticker(exchange, input_symbol_pairs, print_res, time)

        # Ensuring the correct type was returned
        self.assertEqual(list, type(res))
        self.assertTrue(len(res) > 0)

        # Ensuring the correct format was returned
        dp = res[-1]
        self.assertEqual(dict, type(dp))

        # Checking to see if the response was in one of the channels (the user may request multiple)
        possible_channels = []
        for pair in output_symbol_pairs:
            possible_channels.append(f'{exchange}.spot.ticker.{pair}')

        # Picking five random data points to see if they are from the requested channels
        for i in range(5):
            rand = random.randint(0, len(res) - 1)
            self.assertEqual(res[rand]['assetClass'], 'spot')
            self.assertTrue(res[rand]['channel'] in possible_channels)
        
    def book(self, exchange, input_symbol_pair, output_symbol_pairs, print_res, time):
 
        gqpy = gq.GoQuant()

        res = gqpy.live.book(exchange, input_symbol_pair, print_res, time)

        self.assertEqual(list, type(res))
        self.assertTrue(len(res) > 0)

        dp = res[-1]
        self.assertEqual(dict, type(dp))

        # Checking to see if the response was in one of the channels (the user may request multiple)
        possible_channels = []
        for pair in output_symbol_pairs:
            possible_channels.append(f'{exchange}.spot.book.{pair}')

        # Picking five random data points to see if they are from the requested channels
        for i in range(5):
            rand = random.randint(0, len(res) - 1)
            self.assertEqual(res[rand]['assetClass'], 'spot')
            self.assertTrue(res[rand]['channel'] in possible_channels)

    def trade(self, exchange, input_symbol_pair, output_symbol_pairs, print_res, time):

        gqpy = gq.GoQuant()

        res = gqpy.live.trade(exchange, input_symbol_pair, print_res, time)

        self.assertEqual(list, type(res))
        self.assertTrue(len(res) > 0)

        dp = res[-1]
        self.assertEqual(dict, type(dp))

        # Checking to see if the response was in one of the channels (the user may request multiple)
        possible_channels = []
        for pair in output_symbol_pairs:
            possible_channels.append(f'{exchange}.spot.trade.{pair}')

        # Picking five random data points to see if they are from the requested channels
        for i in range(5):
            rand = random.randint(0, len(res) - 1)
            self.assertEqual(res[rand]['assetClass'], 'spot')
            self.assertTrue(res[rand]['channel'] in possible_channels)


# TODO: Remove this file, symbol exchange calls have been moved to the API

# Importing external libraries
import requests

def binance():

        # Getting exchange info from Binance's public API endpoint
        res = requests.get('https://api.binance.com/api/v3/exchangeInfo')
        symbol_pair_list = res.json()['symbols']
        
        symbols = []
        for symbol in symbol_pair_list:
            symbols.append(symbol['symbol'].lower())

        return symbols

def bitfinex():

    # Getting exchange info from Binance's public API endpoint
    res = requests.get('https://api.bitfinex.com/v1/symbols_details')
    symbol_pair_list = res.json()
    
    symbols = []
    for symbol in symbol_pair_list:
        if ':' not in symbol['pair']: symbols.append(symbol['pair'].lower())

    return symbols

def bitstamp():

    res = requests.get('https://www.bitstamp.net/api/v2/trading-pairs-info/')
    symbol_pair_list = res.json()
    
    symbols = []
    for symbol in symbol_pair_list:
        symbols.append(symbol['url_symbol'].lower())

    return symbols

def bybit():

    # Getting exchange info from Binance's public API endpoint
    res = requests.get('https://api.bybit.com/derivatives/v3/public/instruments-info')
    symbol_pair_list = res.json()['result']['list']
    
    symbols = []
    for symbol in symbol_pair_list:
        symbols.append(symbol['symbol'].lower())

    return symbols

def coinbase():

    # Getting exchange info from Binance's public API endpoint
    res = requests.get('https://api.exchange.coinbase.com/products/')
    symbol_pair_list = res.json()
    
    symbols = []
    for symbol in symbol_pair_list:
        symbols.append(symbol['id'].lower())

    return symbols

def huobi():

    # Getting exchange info from Binance's public API endpoint
    res = requests.get('https://api.huobi.pro/v1/common/symbols')
    symbol_pair_list = res.json()['data']

    symbols = []
    for symbol in symbol_pair_list:
        symbols.append(symbol['symbol'].lower())

    return symbols

def okex():

    # Getting exchange info from Binance's public API endpoint
    res = requests.get('https://www.okx.com/api/v5/public/instruments?instType=SPOT')
    symbol_pair_list = res.json()['data']

    symbols = []
    for symbol in symbol_pair_list:
        symbols.append(symbol['instId'].lower())

    return symbols


# Importing external libraries
import unittest

# Importing libraries we want to test
from tests import Tests

class TestCoinbaseLive(unittest.TestCase):

    def setUp(self):

        self.tests = Tests()

    def test_ticker(self):

        self.tests.ticker('coinbase', ['BTC-USD'], ['BTC-USD'], False, 5)

    def test_ticker_multiple_pairs(self):

        self.tests.ticker('coinbase', ['BTC-USD', 'ETH-BTC', 'RPL-USD'], ['BTC-USD', 'ETH-BTC', 'RPL-USD'], False, 8)

    def test_trade(self):

        self.tests.trade('coinbase', ['BTC-USD'], ['BTC-USD'], False, 5)

    def test_trade_multiple_pairs(self):

        self.tests.trade('coinbase', ['BTC-USD', 'ETH-BTC', 'RPL-USD'], ['BTC-USD', 'ETH-BTC', 'RPL-USD'], False, 8)

    def test_book(self):

        self.tests.book('coinbase', ['BTC-USD'], ['BTC-USD'], False, 5)

    def test_book_multiple_pairs(self):

        self.tests.book('coinbase', ['BTC-USD', 'ETH-BTC', 'RPL-USD'], ['BTC-USD', 'ETH-BTC', 'RPL-USD'], False, 8)

if __name__ == "__main__":
    unittest.main()


# Importing external dependencies
import sys

# Importing internal functions and classes
from .utils.csv_utils import store_to_csv
from .utils.mongo_utils import store_to_mongo
from .storage import Storage

def store_data(storage_obj, data):

    if isinstance(storage_obj, Storage.CsvStorageContext):
        store_to_csv(storage_obj, data)

    elif isinstance(storage_obj, Storage.MongoStorageContext):
        store_to_mongo(storage_obj, data)
        
    else:
        print("Your storage object (the last argument you pass to the live/static function) is not the correct type. It was of type " + type(storage_obj))
        sys.exit(1)


class Storage():

    class CsvStorageContext():

        def __init__(self, file_location):
            self.file_location = file_location

    class MongoStorageContext():

        def __init__(self, query_string, database_name, collection_name):
            self.query_string = query_string
            self.database_name = database_name
            self.collection_name = collection_name


# Importing external libraries
import sys, unittest, os
from pymongo import MongoClient

# Importing libraries we want to test
sys.path.append('../../')
import libs.python.src.gqpy.gqpy as gq

class TestStaticStorage(unittest.TestCase):

    def setUp(self):

        self.gq = gq.GoQuant()

    def test_csv(self):

        path_to_file = 'test_csv_live.txt'

        csv_ctx = self.gq.storage.CsvStorageContext(path_to_file)
        self.gq.live.ticker('binance', ['btcusdt'], False, 5, csv_ctx)

        # Reading the file
        f = open(path_to_file, "r")
        content = f.read()
        self.assertTrue(len(content) > 0)
        f.close()

        if os.path.isfile(path_to_file):
            os.remove(path_to_file)

    def test_mongo(self):

        connection_uri = 'mongodb+srv://admin:1231@cluster0.fiusevn.mongodb.net/?retryWrites=true&w=majority'
        database_name = 'test'
        collection_name = 'btc_usdt_live'

        mongo_ctx = self.gq.storage.MongoStorageContext(connection_uri, database_name, collection_name)
        self.gq.live.ticker('binance', ['btcusdt'], False, 5, mongo_ctx)

        client = MongoClient(connection_uri)
        db = client[database_name]
        col = db[collection_name]
        dp = col.find_one()

        self.assertEqual(dict, type(dp))
        self.assertEqual(dp['assetClass'], 'spot')
        self.assertEqual(dp['channel'], 'binance.spot.ticker.BTCUSDT')

        col.delete_many({})




if __name__ == "__main__":
    unittest.main()


# Importing internal libraries
from .spot.live.live import Live
from .symbols.symbols import Symbols
from .storage.storage import Storage



class GoQuant():

    def __init__(self):

        # Connecting to API V2
        self.__tenant_live_url = 'wss://api.goquant.io/ws'
        # self.static = Static(self.__tenant_historical_url)
        # TODO: Integrate Historical data

        self.live = Live(self.__tenant_live_url)
        self.symbols = Symbols()
        self.storage = Storage()


# Importing external dependencies
from pymongo import MongoClient
import json

# Importing internal libraries
from ..storage import Storage

def store_to_mongo(mongo=Storage.MongoStorageContext(None, None, None), data=None):
    
    doc = json.loads(data)

    client = MongoClient(mongo.query_string)
    db = client[mongo.database_name]
    col = db[mongo.collection_name]
    
    col.insert_one(doc)

    client.close()